@@ -1,4 +1,4 @@
-"""The cimsparql.model module contains the base class CimModel."""
+"""The cimsparql.model module contains the base class Model."""
 
 import asyncio
 import re
@@ -62,6 +62,16 @@ class Model:
         self.config = config or ModelConfig()
         self.mapper = mapper or TypeMapper(self.get_client("Type mapper").service_cfg)
 
+    @property
+    def distinct_clients(self) -> list[GraphDBClient]:
+        obj_ids = set()
+        distinct = []
+        for client in self.clients.values():
+            if id(client) not in obj_ids:
+                obj_ids.add(id(client))
+                distinct.append(client)
+        return distinct
+
     def get_client(self, query_name: str) -> GraphDBClient:
         """
         Return the corret graph db client to execute a query. By default
@@ -138,10 +148,6 @@ class Model:
             {"repo": state_repo, "eq_repo": eq_repo} | substitutes | self.client.prefixes
         )
 
-
-class MultiClientCimModel(Model):
-    """Used to query with sparql queries (typically CIM)."""
-
     @cached_property
     def cim_version(self) -> int:
         return int(re.search("cim(\\d+)", self.client.prefixes["cim"]).group(1))
@@ -220,9 +226,9 @@ class MultiClientCimModel(Model):
            region:
 
         Example:
-            >>> from cimsparql.model import get_cim_model
+            >>> from cimsparql.model import get_single_client_model
             >>> server_url = "127.0.0.1:7200"
-            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model = get_single_client_model(server_url, "LATEST")
             >>> model.wind_generating_units()
 
         """
@@ -254,9 +260,9 @@ class MultiClientCimModel(Model):
            region: Limit to region
 
         Example:
-            >>> from cimsparql.model import get_cim_model
+            >>> from cimsparql.model import get_single_client_model
             >>> server_url = "127.0.0.1:7200"
-            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model = get_single_client_model(server_url, "LATEST")
             >>> model.connections()
         """
         query = self.connections_query(region)
@@ -339,9 +345,9 @@ class MultiClientCimModel(Model):
            region: Limit to region
 
         Example:
-            >>> from cimsparql.model import get_cim_model
+            >>> from cimsparql.model import get_single_client_model
             >>> server_url = "127.0.0.1:7200"
-            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model = get_single_client_model(server_url, "LATEST")
             >>> model.ac_lines()
         """
         query = self.ac_lines_query(region, rate)
@@ -401,9 +407,9 @@ class MultiClientCimModel(Model):
            region: Limit to region
 
         Example:
-            >>> from cimsparql.model import get_cim_model
+            >>> from cimsparql.model import get_single_client_model
             >>> server_url = "127.0.0.1:7200"
-            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model = get_single_client_model(server_url, "LATEST")
             >>> model.two_winding_transformers()
         """
         query = self.two_winding_transformers_query(region, rate)
@@ -430,9 +436,9 @@ class MultiClientCimModel(Model):
         """Query three-winding transformer. Return as three two-winding transformers.
 
         Example:
-            >>> from cimsparql.model import get_cim_model
+            >>> from cimsparql.model import get_single_client_model
             >>> server_url = "127.0.0.1:7200"
-            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model = get_single_client_model(server_url, "LATEST")
             >>> model.two_winding_transformers()
         """
         query = self.three_winding_transformers_query(region, rate)
@@ -526,9 +532,9 @@ class MultiClientCimModel(Model):
            regions: List of regions in database
 
         Example:
-            >>> from cimsparql.model import get_cim_model
+            >>> from cimsparql.model import get_single_client_model
             >>> server_url = "127.0.0.1:7200"
-            >>> model = get_cim_model(server_url, "LATEST")
+            >>> model = get_single_client_model(server_url, "LATEST")
             >>> model.regions
         """
         # TODO: Probably deprecate this property in the future. But keep for now.
@@ -548,6 +554,10 @@ class MultiClientCimModel(Model):
         df = await self.get_table_and_convert(self.hvdc_converter_bidzone_query, index="mrid")
         return HVDCBidzonesSchema(df)
 
+    @property
+    def protective_action_query(self) -> str:
+        return self.template_to_query(templates.PROTECTIVE_ACTION_QUERY)
+
     def add_mrid_query(self, rdf_type: Optional[str] = None, graph: Optional[str] = None) -> str:
         substitutes = {"rdf_type": rdf_type or "?rdf_type", "g": graph or "?g"}
         return self.template_to_query(templates.ADD_MRID_QUERY, substitutes)
@@ -573,7 +583,7 @@ class MultiClientCimModel(Model):
         client.update_query(self.add_mrid_query(rdf_type, graph))
 
 
-class CimModel(MultiClientCimModel):
+class SingleClientModel(Model):
     def __init__(
         self,
         client: GraphDBClient,
@@ -588,7 +598,18 @@ def get_cim_model(
     service_cfg: Optional[ServiceConfig] = None,
     model_cfg: Optional[ModelConfig] = None,
     async_sparql_wrapper: bool = False,
-) -> CimModel:
+) -> SingleClientModel:
+    """
+    Function kept for backward compatibility. Use `get_single_client_model` instead.
+    """
+    return get_single_client_model(service_cfg, model_cfg, async_sparql_wrapper)
+
+
+def get_single_client_model(
+    service_cfg: Optional[ServiceConfig] = None,
+    model_cfg: Optional[ModelConfig] = None,
+    async_sparql_wrapper: bool = False,
+) -> SingleClientModel:
     """Get a CIM Model.
 
 
@@ -599,7 +620,7 @@ def get_cim_model(
             If False, the native SparqlWrapper sends requests via urllib
     """
     c = AsyncGraphDBClient(service_cfg) if async_sparql_wrapper else GraphDBClient(service_cfg)
-    return CimModel(c, model_cfg)
+    return SingleClientModel(c, model_cfg)
 
 
 def query_name(query: str) -> str:
@@ -616,7 +637,7 @@ def get_federated_cim_model(
     tpsvssh_client: GraphDBClient,
     model_cfg: ModelConfig,
     mapper: Optional[TypeMapper] = None,
-) -> MultiClientCimModel:
+) -> Model:
     """
     Return a CIM model where the equipment profile is located in one repo and the topology,
     state variables and steady state hypothesis profile is located in another.
@@ -632,12 +653,14 @@ def get_federated_cim_model(
     exec_from_tpssvssh = (
         "AC Lines",
         "Converters",
+        "Disconnected",
         "Loads",
         "Branch node withdraw",
         "DC Active Power Flow",
         "Series compensators",
         "SV branch",
+        "Power flow",
     )
     for query in exec_from_tpssvssh:
         clients[query] = tpsvssh_client
-    return MultiClientCimModel(clients, model_cfg, mapper)
+    return Model(clients, model_cfg, mapper)
